import { Component, Input, OnChanges } from "@angular/core";

import { I18nService } from "../../services/internationalisation.service";
import { fv, longestVarParam } from "../../util";
import { MultiDimensionResults } from "../../results/multidimension-results";
import { PrebarrageResults } from "../../results/prebarrage-results";

import { CalculatorType, PbBassin, PbCloison, Structure, VariatedDetails } from "jalhyd";

@Component({
    selector: "variable-results-selector",
    templateUrl: "./variable-results-selector.component.html",
    styleUrls: [
        "./variable-results-selector.component.scss"
    ]
})
export class VariableResultsSelectorComponent implements OnChanges {

    /** résultats non mis en forme */
    @Input()
    private results: MultiDimensionResults;

    /** détails des paramètres qui varient dans le Nub associé au formulaire */
    @Input()
    private variatedParameters: VariatedDetails[];

    /** valeur en cours */
    private _selectedValue = 0;

    /** size of the longest variable value */
    private size = 0;

    /** inferred extended values list for each variating parameter */
    private varValues = [];

    /* @Output()
    protected indexChange = new EventEmitter(); */

    constructor(
        protected intlService: I18nService,
    ) { }

    public ngOnChanges() {
        // rebuild variable parameters values everytime somthing changes
        if (this.variatedParameters) {
            this.varValues = [];
            // find longest list
            const lvp = longestVarParam(this.variatedParameters);
            this.size = lvp.size;
            // get extended values lists for each variable parameter
            for (const v of this.variatedParameters) {
                const vv = [];
                const iter = v.param.getExtendedValuesIterator(this.size);
                while (iter.hasNext) {
                    const nv = iter.next();
                    vv.push(fv(nv.value));
                }
                this.varValues.push(vv);
            }
        }
        // get current variatedIndex even if component was rebuilt
        if (this.results) {
            this._selectedValue = this.results.variableIndex;
        }
    }

    public get hasVariableResults(): boolean {
        return (
            this.results
            && this.results.hasResults
            && this.results.variatedParameters.length > 0
        );
    }

    public get entries(): number[] {
        const ret: number[] = [];
        for (let i = 0; i < this.size; i++) {
            ret.push(i);
        }
        return ret;
    }

    protected entryLabel(index: number): string {
        const kv = [];
        for (let i = 0; i < this.varValues.length; i++) {
            const vv = this.varValues[i];
            const vp = this.results.variatedParameters[i];
            let symbol = vp.param.symbol;
            // is vp a parameter of a child Nub ?
            if (
                vp.param.parentNub
                && vp.param.parentNub !== vp.param.originNub
            ) {
                let childPrefix: string;
                // prefix the label depending on (grand)children type
                switch (vp.param.originNub.calcType) {
                    case CalculatorType.PreBarrage:
                        const pbRes = this.results as PrebarrageResults;
                        if (vp.param.parentNub instanceof Structure) {
                            const struct = vp.param.parentNub as Structure;
                            const wall = struct.parent as PbCloison;
                            const posS = struct.findPositionInParent() + 1;
                            childPrefix = this.intlService.localizeMessage(wall.description);
                            // there might be multiple walls between the same pair of basins
                            if (wall.uid in pbRes.wallsSuffixes) {
                                childPrefix += " (" + pbRes.wallsSuffixes[wall.uid] + ")";
                            }
                            childPrefix += "_" + this.intlService.localizeText("INFO_LIB_STRUCTURE_N_COURT") + posS;
                        } else if (vp.param.parentNub instanceof PbBassin) {
                            const bassin = vp.param.parentNub as PbBassin;
                            childPrefix = this.intlService.localizeMessage(bassin.description);
                        }
                        break;
                    case CalculatorType.MacroRugoCompound:
                        const posMR = vp.param.parentNub.findPositionInParent() + 1;
                        childPrefix = this.intlService.localizeText("INFO_LIB_RADIER_N_COURT") + posMR;
                        break;
                }
                symbol = childPrefix + "_" + symbol;
            }
            kv.push(`${symbol} = ${vv[index]}`);
        }
        return kv.join(", ");
    }

    public get selectedValue(): number {
        return this._selectedValue;
    }

    public set selectedValue(v: number) {
        this.results.variableIndex = v;
        // this.indexChange.emit();
    }

    public get label() {
        return this.intlService.localizeText("INFO_PARAMFIELD_BOUNDARY_CONDITIONS");
    }
}
